@@ -22,6 +22,7 @@ import netrc
 import errno
 from sys import exc_info
 from threading import Event
+
 from offlineimap import folder, imaputil, imapserver, OfflineImapError
 from offlineimap.repository.Base import BaseRepository
 from offlineimap.threadutil import ExitNotifyThread
@@ -565,7 +566,7 @@ class IMAPRepository(BaseRepository):
         """
         return self.getconfboolean('expunge', True)
 
-    def getpassword(self):
+    def getpassword(self, ignore_keyring=False):
         """Return the IMAP password for this repository.
 
         It tries to get passwords in the following order:
@@ -575,6 +576,7 @@ class IMAPRepository(BaseRepository):
         3. read password from file specified in Repository 'remotepassfile'
         4. read password from ~/.netrc
         5. read password from /etc/netrc
+        6. read password from keyring
 
         On success we return the password.
         If all strategies fail we return None."""
@@ -647,9 +649,24 @@ class IMAPRepository(BaseRepository):
                 user = self.getuser()
                 if user is None or user == netrcentry[0]:
                     return netrcentry[2]
-        # No strategy yielded a password!
+
+        # 6. Read password from keyring as the last option
+        if not ignore_keyring:
+            import keyring
+            return keyring.get_password(self.gethost(), self.getuser())
         return None
 
+    def updatepassword(self, password):
+        """
+        This function update provided password into system keyring.
+        None means to remove it.
+        """
+        import keyring
+        if password is None:
+            keyring.delete_password(self.gethost(), self.getuser())
+        else:
+            keyring.set_password(self.gethost(), self.getuser(), password)
+
     def getfolder(self, foldername, decode=True):
         """Return instance of OfflineIMAP representative folder."""
 
@@ -678,7 +695,13 @@ class IMAPRepository(BaseRepository):
         if self.folders is not None:
             return self.folders
         retval = []
-        imapobj = self.imapserver.acquireconnection()
+
+        try:
+            imapobj = self.imapserver.acquireconnection()
+        except OfflineImapError as e:
+            err_msg = f"Error adquiring connection for repository {self.name}: {str(e)}"
+            raise OfflineImapError(err_msg, OfflineImapError.ERROR.REPO, exc_info()[2])
+
         # check whether to list all folders, or subscribed only
         listfunction = imapobj.list
         if self.getconfboolean('subscribedonly', False):
@@ -717,7 +740,7 @@ class IMAPRepository(BaseRepository):
             try:
                 for foldername in self.folderincludes:
                     try:
-                        imapobj.select(imaputil.utf8_IMAP(foldername),
+                        imapobj.select(imaputil.utf8_IMAP(imaputil.foldername_to_imapname(foldername)),
                                        readonly=True)
                     except OfflineImapError as exc:
                         # couldn't select this folderinclude, so ignore folder.
@@ -782,8 +805,7 @@ class IMAPRepository(BaseRepository):
         """Delete a folder on the IMAP server."""
 
         # Folder names with spaces requires quotes
-        if ' ' in foldername:
-            foldername = '"' + foldername + '"'
+        foldername = imaputil.foldername_to_imapname(foldername)
 
         if self.account.utf_8_support:
             foldername = imaputil.utf8_IMAP(foldername)
@@ -825,11 +847,12 @@ class IMAPRepository(BaseRepository):
         folder_paths = [self.getsep().join(parts[:n + 1])
                         for n in range(len(parts))]
         for folder_path in folder_paths:
-            try:
-                self.makefolder_single(folder_path)
-            except OfflineImapError as exc:
-                if '[ALREADYEXISTS]' not in exc.reason:
-                    raise
+            if not imaputil.foldername_to_imapname(folder_path) in [ f.getfullIMAPname() for f in self.getfolders() ] :
+                try:
+                    self.makefolder_single(folder_path)
+                except OfflineImapError as exc:
+                    if '[ALREADYEXISTS]' not in exc.reason:
+                        raise
 
     def makefolder_single(self, foldername):
         """
@@ -847,8 +870,7 @@ class IMAPRepository(BaseRepository):
         imapobj = self.imapserver.acquireconnection()
         try:
             # Folder names with spaces requires quotes
-            if ' ' in foldername:
-                foldername = '"' + foldername + '"'
+            foldername = imaputil.foldername_to_imapname(foldername)
 
             if self.account.utf_8_support:
                 foldername = imaputil.utf8_IMAP(foldername)
@@ -860,6 +882,9 @@ class IMAPRepository(BaseRepository):
                 raise OfflineImapError(msg, OfflineImapError.ERROR.FOLDER)
         finally:
             self.imapserver.releaseconnection(imapobj)
+            if result[0] == 'OK':
+                self.forgetfolders()
+                self.getfolders()
 
 
 class MappedIMAPRepository(IMAPRepository):
